function extractIndicesForStereoPair
% extractIndicesForStereoPair.m
% Purpose:
%     The script extracts all the parameters for stereo image rectification in
%     order to get the largest rectangular image pairs with only valid pixel values. The parameters are
%     sorted in the way images are read in (First pixels of first row, then pixels of 2nd row etc.). 

% Requirements:
%     This script works only in conjunction with the 
%     "Camera Calibration Toolbox for Matlab" from Jean-Yves Bouguet:
%     http://www.vision.caltech.edu/bouguetj/calib_doc/
%     Also required is the modified file 'FindLargestRectanglesPUS.m' (original from
%     Jaroslaw Tuszynski, see
%     http://www.mathworks.com/matlabcentral/fileexchange/28155-inscribed-rectangle/content/FindLargestRectangles.m)

% Procedure:
%     Put the modifieded attached files "rectify_stereo_pair.m" and 'FindLargestRectanglesPUS.m' in your image directory.
%     In matlab add Jean-Yves Bouguet's toolbox in the "set path" menue.
%     In matlab go to the path of your images and calibrate Jean-Yves Bouguet's toolbox the left and right images.
%     run 'rectify_stereo_pair.m'  in your image directory.
%     Out of the new ""rectify_stereo_pair.m" you get the required parameters
%     as mat file into your image directory.
%     Run now this file.

% Output:
%     Output of this script are the parameters for the largest rectangle which contains only
%     image pixels. Select the output format with 'Nformat'.Depending on Nformat the
%     bilinear parameters a1... to a4... are converted to double, uint8, uint16
%     or uint32.


%clear all;
close all;
Nformat=1;%0 for all double,1 for nearest neighbor, 8 for uint8, 16 for uint16, 32 for uint32
Ileft = double(imread('Left01.tif'));
Iright = double(imread('Right01.tif'));


load('StereoParameterDouble');
%Show left image     
   figure('NumberTitle','off','Name','New indices for max rectangle');
   subplot(2,3,1)
   imshow(uint8(Ileft));
   title('left image original');
%Show original right image 
   subplot(2,3,4)
   imshow(uint8(Iright));
   title('right image original');
    drawnow;
%Left image: Find borders of largest rectangle with valid image pixels in left image
   IleftMask = zeros(ny,nx);
   IleftMask(ind_new_left)=1;
   BWleft = im2bw(IleftMask,.5);
   crit = [1 1 0];
   minSize = [floor(ny*.8) floor(nx*.8)];%ny,nx
   [Xleft, Yleft, Wleft, Hleft] = FindLargestRectanglesPUS(BWleft,crit,minSize);
%Right image: Find borders of largest rectangle with valid image pixels in right image
   IrightMask = zeros(ny,nx);
   IrightMask(ind_new_right)=1;
   BWright = im2bw(IrightMask,.5);
   [Xright, Yright, Wright, Hright] = FindLargestRectanglesPUS(BWright,crit,minSize);
%Calulate largest common borders   
   YbothTop=max(Yleft,Yright);%common most top row
   YbothBottom=min(Yleft+Hleft,Yright+Hright);% common most bottom row
   Hboth=YbothBottom-YbothTop+1;%common height
   titleTxt='common height'; 
%Uncomment following 8 lines if left and right images shall have same size and position 
   XbothLeft=max(Xleft,Xright);%common most left column
   XbothRight=min(Xleft+Wleft-1,Xright+Wright-1);%common most right column
   Wboth=XbothRight-XbothLeft+1;%Wboth=min(Wleft,Wright);
   Xleft=XbothLeft;
   Xright=XbothLeft;
   Wleft=Wboth;
   Wright=Wboth;
   titleTxt='common rectangle and position'; 
%
%Calibrate and rectify left image
   IleftRect = 255*ones(ny,nx);
   IleftRect(ind_new_left) = (a1_left .* Ileft(ind_1_left) + a2_left .* Ileft(ind_2_left) + ...
       a3_left .* Ileft(ind_3_left) + a4_left .* Ileft(ind_4_left));
%Show callibrated and rectified left image (full size)  
    subplot(2,3,2);
    imshow(uint8(IleftRect));
    title(sprintf(['left image calibrated and rectified (full precision).\nred: max rectangle, green: ' titleTxt]));
    rectangle('Position',[Xleft, Yleft, Wleft, Hleft], 'EdgeColor','r', 'LineWidth',2);%max rectangle left
    rectangle('Position',[Xleft, YbothTop, Wleft, Hboth], 'EdgeColor','g', 'LineWidth',1);%common height
%Calibrate and rectify right image   
   IrightRect = 255*ones(ny,nx);
   IrightRect(ind_new_right) = (a1_right .* Iright(ind_1_right) + a2_right .* Iright(ind_2_right) +...
       a3_right .* Iright(ind_3_right) + a4_right .* Iright(ind_4_right));
    
%Show callibrated and rectified right image (full size)     
    subplot(2,3,5);
    imshow(uint8(IrightRect));
    title(sprintf(['right image calibrated and rectified (full precision).\nred: max rectangle, green: ' titleTxt]));
    rectangle('Position',[Xright, Yright, Wright, Hright], 'EdgeColor','r', 'LineWidth',1);%max rectangle right
    rectangle('Position',[Xright, YbothTop, Wright, Hboth], 'EdgeColor','g', 'LineWidth',1);%common height  
    drawnow;
%Left image: Calculate new indices for only max rectangle and sort them
    indnew=0;
    Xleftr=Xleft+Wleft-1;%right end
    %Yleftb=Yleft+Hboth-1;%bottom end
    leftAll=zeros(Hboth*Wleft,11);%array for extracted parameters
    for indi=1:size(ind_new_left)%Loop through each index
        coli=floor((ind_new_left(indi)-1)/ny);%evaluate column of rectified image
        rowi=ind_new_left(indi)-coli*ny;%evaluate row of rectified image
        coli=coli+1;
        %Check if pixel is within rectangle, if yes put parameters in in new array
        if (coli>=Xleft) && (coli<=Xleftr) && (rowi>=YbothTop) && rowi<=YbothBottom
            %Pixel is within rectangle
            indnew=indnew+1;
            indNi=(coli-Xleft)*Hboth+rowi-YbothTop+1;% make new linear index for rectangle image
            leftAll(indnew,1)=indNi;
            leftAll(indnew,2)=ind_1_left(indi);%copy index in new array
            leftAll(indnew,3)=ind_2_left(indi);
            leftAll(indnew,4)=ind_3_left(indi);
            leftAll(indnew,5)=ind_4_left(indi);
            leftAll(indnew,6)=a1_left(indi);
            leftAll(indnew,7)=a2_left(indi);
            leftAll(indnew,8)=a3_left(indi);
            leftAll(indnew,9)=a4_left(indi);
        end
    end
%Sort pixel position from top left to bottom right
    col1org=floor((leftAll(:,2))/ny);%get column from index
    row1org=leftAll(:,2)-col1org*ny;%get row from index
    col1org=col1org+1;%adjust since first column is 1 and not zero
    leftAll(:,10)=row1org;%put results in array
    leftAll(:,11)=col1org;
    leftAll=sortrows(leftAll,[10 11]);%sort by row and then by column
%Put back in format as previously used (single rows)  
    ind_new_leftNi=leftAll(:,1)';
    ind_1_leftNi=leftAll(:,2)';
    ind_2_leftNi=leftAll(:,3)';
    ind_3_leftNi=leftAll(:,4)';
    ind_4_leftNi=leftAll(:,5)';
    a1_leftNi=leftAll(:,6)';
    a2_leftNi=leftAll(:,7)';
    a3_leftNi=leftAll(:,8)';
    a4_leftNi=leftAll(:,9)';
    clear leftAll;%free up memory

%Right image: Calculate new indices for only max rectangle and sort them
    indnew=0;
    Xrightr=Xright+Wright-1;%right end
    %Yrightb=Yright+Hboth-1;%bottom end
    rightAll=zeros(Hboth*Wright,11);
    for indi=1:size(ind_new_right)
        coli=floor((ind_new_right(indi)-1)/ny);
        rowi=ind_new_right(indi)-coli*ny;
        coli=coli+1;
        if (coli>=Xright) && (coli<=Xrightr) && (rowi>=YbothTop) && rowi<=YbothBottom
            indnew=indnew+1;
            indNi=(coli-Xright)*Hboth+rowi-YbothTop+1;
            
            rightAll(indnew,1)=indNi;
            rightAll(indnew,2)=ind_1_right(indi);
            rightAll(indnew,3)=ind_2_right(indi);
            rightAll(indnew,4)=ind_3_right(indi);
            rightAll(indnew,5)=ind_4_right(indi);
            rightAll(indnew,6)=a1_right(indi);
            rightAll(indnew,7)=a2_right(indi);
            rightAll(indnew,8)=a3_right(indi);
            rightAll(indnew,9)=a4_right(indi);
        end
    end
    col1org=floor((rightAll(:,2))/ny);
    row1org=rightAll(:,2)-col1org*ny;
    col1org=col1org+1;
    rightAll(:,10)=row1org;
    rightAll(:,11)=col1org;
    rightAll=sortrows(rightAll,[10 11]);
    ind_new_rightNi=rightAll(:,1)';
    ind_1_rightNi=rightAll(:,2)';
    ind_2_rightNi=rightAll(:,3)';
    ind_3_rightNi=rightAll(:,4)';
    ind_4_rightNi=rightAll(:,5)';
    a1_rightNi=rightAll(:,6)';
    a2_rightNi=rightAll(:,7)';
    a3_rightNi=rightAll(:,8)';
    a4_rightNi=rightAll(:,9)';
    clear rightAll;
%Show right image calculated with New Indices 
    IrightRectNi = 255*ones(Hboth,Wright);
    IrightRectNi(ind_new_rightNi) = (a1_rightNi .* Iright(ind_1_rightNi) + a2_rightNi ...
         .* Iright(ind_2_rightNi) + a3_rightNi .* Iright(ind_3_rightNi) + a4_rightNi .* Iright(ind_4_rightNi));                      
    subplot(2,3,6);
    imshow(uint8(IrightRectNi));
    title('right image calculated by new indices');  
    set(gcf, 'Position', get(0,'Screensize')); % Maximize figure. 
%Save parameters for rectangle images for later use. 
%recomended for matlab: double format (format=0)
%for FPGA try uint8 or uint16, Only a-parameters are converted to uint8 or
%uint16 other parameters are converted to uint32
    %Nformat=32;%0 for double, 8 for uint8, 16 for uint16, 32 for uint32
    if Nformat~=0
    %convert indices and other parameters to uint32 (does not change precision) 
         Nformat=uint32(Nformat);
         Hboth=uint32(Hboth);
         Wleft=uint32(Wleft);
         Wright=uint32(Wright);
         ind_new_leftNi=uint32(ind_new_leftNi);
         ind_1_leftNi=uint32(ind_1_leftNi);
         ind_2_leftNi=uint32(ind_2_leftNi);
         ind_3_leftNi=uint32(ind_3_leftNi);
         ind_4_leftNi=uint32(ind_4_leftNi);
         ind_1_rightNi=uint32(ind_1_rightNi);
         ind_2_rightNi=uint32(ind_2_rightNi);
         ind_3_rightNi=uint32(ind_3_rightNi);
         ind_4_rightNi=uint32(ind_4_rightNi);  
    end
    switch Nformat
        case 0 %double
            filename='StereoParameterNiDouble';
            factor=1;
           titleleftTxt='double, left image calculate by new indices'; 
           titlerightTxt='double, right image calculate by new indices';               
            disp('Saved as StereoParameterNiDouble.mat');            
        case 1 %nearest neighbor
           filename='StereoParameterNiNNeighbor';
           factor=1;
           %left:
           [~,size_a ]=size(a1_leftNi);
           a_all=zeros(size_a,5);
           a_all(:,1)=a1_leftNi';
           a_all(:,2)=a2_leftNi';
           a_all(:,3)=a3_leftNi';
           a_all(:,4)=a4_leftNi';

           [xx, ind]=max(a_all(:,1:4),[],2);
           a_all(:,5)=ind;% index of biggest value of a (nearest neighbor)
           a_all(:,1:4)=0;%clear all a
           %and set nearesr neighbor to one:
           for i=1:size_a 
              a_all(i,a_all(i,5))=1;
            end             
           
           a1_leftNi=uint8(a_all(:,1))';
           a2_leftNi=uint8(a_all(:,2))';
           a3_leftNi=uint8(a_all(:,3))';
           a4_leftNi=uint8(a_all(:,4))'; 
          
        %Do the same for right values
           a_all=zeros(size_a,5);
           a_all(:,1)=a1_rightNi';
           a_all(:,2)=a2_rightNi';
           a_all(:,3)=a3_rightNi';
           a_all(:,4)=a4_rightNi';

           [xx, ind]=max(a_all(:,1:4),[],2);
           a_all(:,5)=ind;% index of biggest value of a (nearest neighbor)
           a_all(:,1:4)=0;%clear all a
           %and set nearesr neighbor to one:
           for i=1:size_a 
              a_all(i,a_all(i,5))=1;
            end             
           
           a1_rightNi=uint8(a_all(:,1))';
           a2_rightNi=uint8(a_all(:,2))';
           a3_rightNi=uint8(a_all(:,3))';
           a4_rightNi=uint8(a_all(:,4))';
           
           titleleftTxt='Nearest neighbor, left image calculate by new indices'; 
           titlerightTxt='Nearest neighbor, right image calculate by new indices'; 
           disp('Saved as StereoParameterNiNNeighbor.mat');
            
        case 8
           filename='StereoParameterNiUint8'; 
           factor=2^7;
           %in the case of double this must be assured: a1+a2+a3+a4=1
           %in the case of uint8 this must be assured: a1+a2+a3+a4=128
           %Due to rounding effects with uint8 this does not hold for all
           %values
           %left:
           [~,size_a ]=size(a1_leftNi);
           a_all=zeros(size_a,8);
           a_all(:,1)=double(uint8(round(a1_leftNi*factor)))';
           a_all(:,2)=double(uint8(round(a2_leftNi*factor)))';
           a_all(:,3)=double(uint8(round(a3_leftNi*factor)))';
           a_all(:,4)=double(uint8(round(a4_leftNi*factor)))';

           a_all=correct_a(a_all,factor,size_a);
           
           a1_leftNi=uint8(a_all(:,1))';
           a2_leftNi=uint8(a_all(:,2))';
           a3_leftNi=uint8(a_all(:,3))';
           a4_leftNi=uint8(a_all(:,4))'; 
        %Do the same for right values
            a_all=zeros(size_a,8);
           a_all(:,1)=double(uint8(round(a1_rightNi*factor)))';
           a_all(:,2)=double(uint8(round(a2_rightNi*factor)))';
           a_all(:,3)=double(uint8(round(a3_rightNi*factor)))';
           a_all(:,4)=double(uint8(round(a4_rightNi*factor)))';
           
           a_all=correct_a(a_all,factor,size_a);
            
           a1_rightNi=uint8(a_all(:,1))';
           a2_rightNi=uint8(a_all(:,2))';
           a3_rightNi=uint8(a_all(:,3))';
           a4_rightNi=uint8(a_all(:,4))'; 
           
           titleleftTxt='uint8, left image calculate by new indices'; 
           titlerightTxt='uint8, right image calculate by new indices';           
           disp('Saved as StereoParameterNiUint8.mat');

           
        case 16
           filename='StereoParameterNiUint16'; 
          factor=2^15;
           [~,size_a ]=size(a1_leftNi);
           a_all=zeros(size_a,8);
           a_all(:,1)=double(uint16(round(a1_leftNi*factor)))';
           a_all(:,2)=double(uint16(round(a2_leftNi*factor)))';
           a_all(:,3)=double(uint16(round(a3_leftNi*factor)))';
           a_all(:,4)=double(uint16(round(a4_leftNi*factor)))';

           a_all=correct_a(a_all,factor,size_a);
           
           a1_leftNi=uint16(a_all(:,1))';
           a2_leftNi=uint16(a_all(:,2))';
           a3_leftNi=uint16(a_all(:,3))';
           a4_leftNi=uint16(a_all(:,4))'; 
        %Do the same for right values
            a_all=zeros(size_a,8);
           a_all(:,1)=double(uint16(round(a1_rightNi*factor)))';
           a_all(:,2)=double(uint16(round(a2_rightNi*factor)))';
           a_all(:,3)=double(uint16(round(a3_rightNi*factor)))';
           a_all(:,4)=double(uint16(round(a4_rightNi*factor)))';
           
           a_all=correct_a(a_all,factor,size_a);
           
           a1_rightNi=uint16(a_all(:,1))';
           a2_rightNi=uint16(a_all(:,2))';
           a3_rightNi=uint16(a_all(:,3))';
           a4_rightNi=uint16(a_all(:,4))'; 

           titleleftTxt='uint16, left image calculate by new indices'; 
           titlerightTxt='uint16, right image calculate by new indices';              
           disp('Saved as StereoParameterNiUint16.mat');       
        case 32
           filename='StereoParameterNiUint32'; 
           factor=2^31;
           [~,size_a ]=size(a1_leftNi);
           a_all=zeros(size_a,8);
           a_all(:,1)=double(uint32(round(a1_leftNi*factor)))';
           a_all(:,2)=double(uint32(round(a2_leftNi*factor)))';
           a_all(:,3)=double(uint32(round(a3_leftNi*factor)))';
           a_all(:,4)=double(uint32(round(a4_leftNi*factor)))';
           
           a_all=correct_a(a_all,factor,size_a);
      
           a1_leftNi=uint32(a_all(:,1))';
           a2_leftNi=uint32(a_all(:,2))';
           a3_leftNi=uint32(a_all(:,3))';
           a4_leftNi=uint32(a_all(:,4))'; 
        %Do the same for right values
            a_all=zeros(size_a,8);
           a_all(:,1)=double(uint32(round(a1_rightNi*factor)))';
           a_all(:,2)=double(uint32(round(a2_rightNi*factor)))';
           a_all(:,3)=double(uint32(round(a3_rightNi*factor)))';
           a_all(:,4)=double(uint32(round(a4_rightNi*factor)))';
           
           a_all=correct_a(a_all,factor,size_a);
      
           a1_rightNi=uint32(a_all(:,1))';
           a2_rightNi=uint32(a_all(:,2))';
           a3_rightNi=uint32(a_all(:,3))';
           a4_rightNi=uint32(a_all(:,4))';
           titleleftTxt='uint32, left image calculate by new indices'; 
           titlerightTxt='uint32, right image calculate by new indices';              
           disp('Saved as StereoParameterNiUint32.mat');             
        otherwise
            error(['Wrong numberformat!!:' num2str(Nformat) ' Select 0,1,8,16 or 32 for Nformat']);
    end
 %Show left image calculated with New Indices    
            IleftRectNi = 255*ones(Hboth,Wleft);

             IleftRectNi(ind_new_leftNi) = ((double(a1_leftNi) .* Ileft(ind_1_leftNi) +...
                 double(a2_leftNi) .* Ileft(ind_2_leftNi) +...
                 double(a3_leftNi) .* Ileft(ind_3_leftNi) +...
                 double(a4_leftNi) .* Ileft(ind_4_leftNi)))/factor;                      
            subplot(2,3,3);

            imshow(uint8(IleftRectNi));
            title(titleleftTxt); 
 %Show right image calculated with New Indices    
            IrightRectNi = 255*ones(Hboth,Wright);

             IrightRectNi(ind_new_rightNi) = ((double(a1_rightNi) .* Iright(ind_1_rightNi) +...
                 double(a2_rightNi) .* Iright(ind_2_rightNi) +...
                 double(a3_rightNi) .* Iright(ind_3_rightNi) +...
                 double(a4_rightNi) .* Iright(ind_4_rightNi)))/factor;                     
            subplot(2,3,6);

            imshow(uint8(IrightRectNi));
            title(titlerightTxt);                    
%Save data  
    save(filename,'Hboth','Wleft','Wright','ind_new_leftNi',...
        'ind_1_leftNi','ind_2_leftNi','ind_3_leftNi','ind_4_leftNi',...
        'a1_leftNi','a2_leftNi','a3_leftNi','a4_leftNi',...
        'ind_new_rightNi','ind_1_rightNi','ind_2_rightNi','ind_3_rightNi','ind_4_rightNi',...
        'a1_rightNi','a2_rightNi','a3_rightNi','a4_rightNi');
end
%%
function  a_all=correct_a(a_all,factor,size_a)
%evaluate difference of (a+a2+a3+a4)-128 (or -2^16, -2^32)
           a_all(:,5)=(a_all(:,1)+a_all(:,2)+a_all(:,3)+a_all(:,4))-factor;
           a_all(:,6)=double(uint8(a_all(:,5)));%for too big sum, cut negative values
           a_all(:,5)=double(uint8(-a_all(:,5)));%too small sum,, cut positive values
           [xx, ind]=max(a_all(:,1:4),[],2);
           a_all(:,7)=ind;% index of biggest value of a
           [xx, ind]=min(a_all(:,1:4),[],2);
           a_all(:,8)=ind;% index of smallest value of a
           for i=1:size_a 
              a_all(i,a_all(i,7))=a_all(i,a_all(i,7))-a_all(i,6);% correct and prevents overflow (>factor)
              a_all(i,a_all(i,8))=a_all(i,a_all(i,8))+a_all(i,5);%correct too small values
           end
%            for i=1:size_a 
%                %correct, but try to keep weight of closed pixel as large as
%                %possible
%               if a_all(i,6)>0 && a_all(i,a_all(i,8))>=a_all(i,6)% correct and prevents overflow (>factor)
%                   a_all(i,a_all(i,8))=a_all(i,a_all(i,8))-a_all(i,6);%substract from most distant pixel
%               elseif a_all(i,6)>0  
%               a_all(i,a_all(i,7))=a_all(i,a_all(i,7))-a_all(i,6);%substract from most close pixel
%               end
%               if a_all(i,5)>0 && a_all(i,a_all(i,7))+a_all(i,5)<=factor
%                   a_all(i,a_all(i,7))=a_all(i,a_all(i,7))+a_all(i,5);%correct too small values
%               elseif a_all(i,5)>0
%                     a_all(i,a_all(i,8))=a_all(i,a_all(i,8))+a_all(i,5);
%               end
%            end 
end
